
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <ctype.h>


unsigned long CalcTableChecksum(unsigned long* table, unsigned long length);



main(int argc, char* argv[])
{
	if (argc != 2) {
		printf("Usage: ttc2ttf filename\n");
		exit(2);
	}

	char*		filename = argv[1];
	FILE*		in_file = fopen(filename, "rt");
	if (in_file == NULL) {
		printf("\"%s\" not found\n", filename);
		exit(2);
	}
	struct	stat	info;
	fstat(fileno(in_file), &info);
	char*			buffer = (char*) malloc(info.st_size);
	fread(buffer, sizeof(char), info.st_size, in_file);
	fclose(in_file);
	
	if (strlen(filename) > 4) {
		char	tmp[5];
		strcpy(tmp, &filename[strlen(filename) - 4]);
		for (long i = 0; i < 4; i++)
			tmp[i] = toupper(tmp[i]);
		if (strcmp(tmp, ".TTC") == 0)
			filename[strlen(filename) - 4] = 0;
	}





	if (*(long*) buffer != 'ttcf')
	{
		char	out_filename[256];
		sprintf(out_filename, "%s.ttf", filename);
		FILE*	out_file = fopen(out_filename, "wt");
		fwrite(buffer, sizeof(char), info.st_size, out_file);
		fclose(out_file);
	}
	else
	{
		unsigned long	ttf_count = *(unsigned long*) &buffer[0x08];
		unsigned long*	ttf_offset_array = (unsigned long*) &buffer[0x0C];

		for (unsigned long i = 0; i < ttf_count; i++) {
			unsigned long		table_header_offset = ttf_offset_array[i];
			unsigned short		table_count = *(unsigned short*) &buffer[table_header_offset + 0x04];
			unsigned long		header_length = 0x0C + table_count * 0x10;
			unsigned long		table_length = 0;
			for (unsigned short j = 0; j < table_count; j++) {
				unsigned long	length = *(unsigned long*) &buffer[table_header_offset + 0x0C + 0x0C + j * 0x10];
				table_length += (length + 3) & ~3;
			}

			unsigned long	total_length = header_length + table_length;
			char*			new_buffer = (char*) malloc(total_length);
			unsigned long	pad = 0;
			memcpy(new_buffer, &buffer[table_header_offset], header_length);
			unsigned long	current_offset = header_length;
			for (unsigned short j = 0; j < table_count; j++) {
				unsigned long	offset = *(unsigned long*) &buffer[table_header_offset + 0x0C + 0x08 + j * 0x10];
				unsigned long	length = *(unsigned long*) &buffer[table_header_offset + 0x0C + 0x0C + j * 0x10];
				*(unsigned long*) &new_buffer[0x0C + 0x08 + j * 0x10] = current_offset;
				memcpy(&new_buffer[current_offset], &buffer[offset], length);
				memcpy(&new_buffer[current_offset + length], &pad, ((length + 3) & ~3) - length);
				*(unsigned long*) &new_buffer[0x0C + 0x04 + j * 0x10] = CalcTableChecksum((unsigned long*) &new_buffer[current_offset], length);
				current_offset += (length + 3) & ~3;
			}


			char	out_filename[256];
			sprintf(out_filename, "%s%d.ttf", filename, i);
			FILE*	out_file = fopen(out_filename, "wt");
			fwrite(new_buffer, sizeof(char), total_length, out_file);
			fclose(out_file);
			free(new_buffer);
		}
	}

	free(buffer);
}

unsigned long
CalcTableChecksum(unsigned long* table, unsigned long length)
{
	unsigned long	sum = 0L;
	unsigned long*	end_ptr = table + ((length + 3) & ~3) / sizeof(unsigned long);
	while (table < end_ptr)
		sum += *table++;
	return sum;
}

